@@ -29,16 +29,15 @@ pub mod scalar;
 pub mod stats;
 mod table_reference;
 pub mod test_util;
+pub mod tree_node;
 pub mod utils;
 
-use arrow::compute::SortOptions;
 pub use column::Column;
 pub use dfschema::{DFField, DFSchema, DFSchemaRef, ExprSchema, ToDFSchema};
 pub use error::{
     field_not_found, unqualified_field_not_found, DataFusionError, Result, SchemaError,
     SharedResult,
 };
-pub use parsers::parse_interval;
 pub use scalar::{ScalarType, ScalarValue};
 pub use stats::{ColumnStatistics, Statistics};
 pub use table_reference::{OwnedTableReference, ResolvedTableReference, TableReference};
@@ -68,12 +67,3 @@ macro_rules! downcast_value {
         })?
     }};
 }
-
-/// Computes the "reverse" of given `SortOptions`.
-// TODO: If/when arrow supports `!` for `SortOptions`, we can remove this.
-pub fn reverse_sort_options(options: SortOptions) -> SortOptions {
-    SortOptions {
-        descending: !options.descending,
-        nulls_first: !options.nulls_first,
-    }
-}
