@@ -500,8 +500,7 @@ mod tests {
         let expected = "\
         Explain\
         \n  CreateView: Bare { table: \"xyz\" }\
-        \n    Projection: abc.column1, abc.column2, abc.column3\
-        \n      TableScan: abc projection=[column1, column2, column3]";
+        \n    TableScan: abc projection=[column1, column2, column3]";
         assert_eq!(expected, actual);
 
         let dataframe = session_ctx
@@ -512,9 +511,8 @@ mod tests {
         let expected = "\
         Explain\
         \n  CreateView: Bare { table: \"xyz\" }\
-        \n    Projection: abc.column1, abc.column2, abc.column3\
-        \n      Filter: abc.column2 = Int64(5)\
-        \n        TableScan: abc projection=[column1, column2, column3]";
+        \n    Filter: abc.column2 = Int64(5)\
+        \n      TableScan: abc projection=[column1, column2, column3]";
         assert_eq!(expected, actual);
 
         let dataframe = session_ctx
@@ -525,9 +523,8 @@ mod tests {
         let expected = "\
         Explain\
         \n  CreateView: Bare { table: \"xyz\" }\
-        \n    Projection: abc.column1, abc.column2\
-        \n      Filter: abc.column2 = Int64(5)\
-        \n        TableScan: abc projection=[column1, column2]";
+        \n    Filter: abc.column2 = Int64(5)\
+        \n      TableScan: abc projection=[column1, column2]";
         assert_eq!(expected, actual);
 
         Ok(())
