@@ -29,20 +29,20 @@ use crate::physical_plan::joins::{
 use crate::physical_plan::projection::ProjectionExec;
 use crate::physical_plan::repartition::RepartitionExec;
 use crate::physical_plan::sorts::sort::SortOptions;
-use crate::physical_plan::tree_node::TreeNodeRewritable;
 use crate::physical_plan::windows::WindowAggExec;
 use crate::physical_plan::Partitioning;
 use crate::physical_plan::{with_new_children_if_necessary, Distribution, ExecutionPlan};
 use arrow::datatypes::SchemaRef;
+use datafusion_common::tree_node::{Transformed, TreeNode, VisitRecursion};
 use datafusion_expr::logical_plan::JoinType;
 use datafusion_physical_expr::equivalence::EquivalenceProperties;
 use datafusion_physical_expr::expressions::Column;
 use datafusion_physical_expr::expressions::NoOp;
+use datafusion_physical_expr::utils::map_columns_before_projection;
 use datafusion_physical_expr::{
     expr_list_eq_strict_order, normalize_expr_with_equivalence_properties, AggregateExpr,
     PhysicalExpr,
 };
-use std::collections::HashMap;
 use std::sync::Arc;
 
 /// The EnforceDistribution rule ensures that distribution requirements are met
@@ -81,15 +81,13 @@ impl PhysicalOptimizerRule for EnforceDistribution {
             plan
         };
         // Distribution enforcement needs to be applied bottom-up.
-        new_plan.transform_up(&{
-            |plan| {
-                let adjusted = if !top_down_join_key_reordering {
-                    reorder_join_keys_to_inputs(plan)?
-                } else {
-                    plan
-                };
-                Ok(Some(ensure_distribution(adjusted, target_partitions)?))
-            }
+        new_plan.transform_up(&|plan| {
+            let adjusted = if !top_down_join_key_reordering {
+                reorder_join_keys_to_inputs(plan)?
+            } else {
+                plan
+            };
+            ensure_distribution(adjusted, target_partitions)
         })
     }
 
@@ -146,10 +144,10 @@ impl PhysicalOptimizerRule for EnforceDistribution {
 ///
 fn adjust_input_keys_ordering(
     requirements: PlanWithKeyRequirements,
-) -> Result<Option<PlanWithKeyRequirements>> {
+) -> Result<Transformed<PlanWithKeyRequirements>> {
     let parent_required = requirements.required_key_ordering.clone();
     let plan_any = requirements.plan.as_any();
-    if let Some(HashJoinExec {
+    let transformed = if let Some(HashJoinExec {
         left,
         right,
         on,
@@ -174,13 +172,13 @@ fn adjust_input_keys_ordering(
                             *null_equals_null,
                         )?) as Arc<dyn ExecutionPlan>)
                     };
-                Ok(Some(reorder_partitioned_join_keys(
+                Some(reorder_partitioned_join_keys(
                     requirements.plan.clone(),
                     &parent_required,
                     on,
                     vec![],
                     &join_constructor,
-                )?))
+                )?)
             }
             PartitionMode::CollectLeft => {
                 let new_right_request = match join_type {
@@ -198,17 +196,15 @@ fn adjust_input_keys_ordering(
                 };
 
                 // Push down requirements to the right side
-                Ok(Some(PlanWithKeyRequirements {
+                Some(PlanWithKeyRequirements {
                     plan: requirements.plan.clone(),
                     required_key_ordering: vec![],
                     request_key_ordering: vec![None, new_right_request],
-                }))
+                })
             }
             PartitionMode::Auto => {
                 // Can not satisfy, clear the current requirements and generate new empty requirements
-                Ok(Some(PlanWithKeyRequirements::new(
-                    requirements.plan.clone(),
-                )))
+                Some(PlanWithKeyRequirements::new(requirements.plan.clone()))
             }
         }
     } else if let Some(CrossJoinExec { left, .. }) =
@@ -216,14 +212,14 @@ fn adjust_input_keys_ordering(
     {
         let left_columns_len = left.schema().fields().len();
         // Push down requirements to the right side
-        Ok(Some(PlanWithKeyRequirements {
+        Some(PlanWithKeyRequirements {
             plan: requirements.plan.clone(),
             required_key_ordering: vec![],
             request_key_ordering: vec![
                 None,
                 shift_right_required(&parent_required, left_columns_len),
             ],
-        }))
+        })
     } else if let Some(SortMergeJoinExec {
         left,
         right,
@@ -245,13 +241,13 @@ fn adjust_input_keys_ordering(
                     *null_equals_null,
                 )?) as Arc<dyn ExecutionPlan>)
             };
-        Ok(Some(reorder_partitioned_join_keys(
+        Some(reorder_partitioned_join_keys(
             requirements.plan.clone(),
             &parent_required,
             on,
             sort_options.clone(),
             &join_constructor,
-        )?))
+        )?)
     } else if let Some(AggregateExec {
         mode,
         group_by,
@@ -263,21 +259,19 @@ fn adjust_input_keys_ordering(
     {
         if !parent_required.is_empty() {
             match mode {
-                AggregateMode::FinalPartitioned => Ok(Some(reorder_aggregate_keys(
+                AggregateMode::FinalPartitioned => Some(reorder_aggregate_keys(
                     requirements.plan.clone(),
                     &parent_required,
                     group_by,
                     aggr_expr,
                     input.clone(),
                     input_schema,
-                )?)),
-                _ => Ok(Some(PlanWithKeyRequirements::new(
-                    requirements.plan.clone(),
-                ))),
+                )?),
+                _ => Some(PlanWithKeyRequirements::new(requirements.plan.clone())),
             }
         } else {
             // Keep everything unchanged
-            Ok(None)
+            None
         }
     } else if let Some(ProjectionExec { expr, .. }) =
         plan_any.downcast_ref::<ProjectionExec>()
@@ -287,33 +281,34 @@ fn adjust_input_keys_ordering(
         // Construct a mapping from new name to the the orginal Column
         let new_required = map_columns_before_projection(&parent_required, expr);
         if new_required.len() == parent_required.len() {
-            Ok(Some(PlanWithKeyRequirements {
+            Some(PlanWithKeyRequirements {
                 plan: requirements.plan.clone(),
                 required_key_ordering: vec![],
                 request_key_ordering: vec![Some(new_required.clone())],
-            }))
+            })
         } else {
             // Can not satisfy, clear the current requirements and generate new empty requirements
-            Ok(Some(PlanWithKeyRequirements::new(
-                requirements.plan.clone(),
-            )))
+            Some(PlanWithKeyRequirements::new(requirements.plan.clone()))
         }
     } else if plan_any.downcast_ref::<RepartitionExec>().is_some()
         || plan_any.downcast_ref::<CoalescePartitionsExec>().is_some()
         || plan_any.downcast_ref::<WindowAggExec>().is_some()
     {
-        Ok(Some(PlanWithKeyRequirements::new(
-            requirements.plan.clone(),
-        )))
+        Some(PlanWithKeyRequirements::new(requirements.plan.clone()))
     } else {
         // By default, push down the parent requirements to children
         let children_len = requirements.plan.children().len();
-        Ok(Some(PlanWithKeyRequirements {
+        Some(PlanWithKeyRequirements {
             plan: requirements.plan.clone(),
             required_key_ordering: vec![],
             request_key_ordering: vec![Some(parent_required.clone()); children_len],
-        }))
-    }
+        })
+    };
+    Ok(if let Some(transformed) = transformed {
+        Transformed::Yes(transformed)
+    } else {
+        Transformed::No(requirements)
+    })
 }
 
 fn reorder_partitioned_join_keys<F>(
@@ -492,30 +487,6 @@ fn reorder_aggregate_keys(
     }
 }
 
-fn map_columns_before_projection(
-    parent_required: &[Arc<dyn PhysicalExpr>],
-    proj_exprs: &[(Arc<dyn PhysicalExpr>, String)],
-) -> Vec<Arc<dyn PhysicalExpr>> {
-    let mut column_mapping = HashMap::new();
-    for (expression, name) in proj_exprs.iter() {
-        if let Some(column) = expression.as_any().downcast_ref::<Column>() {
-            column_mapping.insert(name.clone(), column.clone());
-        };
-    }
-    let new_required: Vec<Arc<dyn PhysicalExpr>> = parent_required
-        .iter()
-        .filter_map(|r| {
-            if let Some(column) = r.as_any().downcast_ref::<Column>() {
-                column_mapping.get(column.name())
-            } else {
-                None
-            }
-        })
-        .map(|e| Arc::new(e.clone()) as Arc<dyn PhysicalExpr>)
-        .collect::<Vec<_>>();
-    new_required
-}
-
 fn shift_right_required(
     parent_required: &[Arc<dyn PhysicalExpr>],
     left_columns_len: usize,
@@ -841,11 +812,11 @@ fn new_join_conditions(
 /// takes care of such requirements, we should avoid manually adding data
 /// exchange operators in other places.
 fn ensure_distribution(
-    plan: Arc<dyn crate::physical_plan::ExecutionPlan>,
+    plan: Arc<dyn ExecutionPlan>,
     target_partitions: usize,
-) -> Result<Arc<dyn crate::physical_plan::ExecutionPlan>> {
+) -> Result<Transformed<Arc<dyn ExecutionPlan>>> {
     if plan.children().is_empty() {
-        return Ok(plan);
+        return Ok(Transformed::No(plan));
     }
 
     let required_input_distributions = plan.required_input_distribution();
@@ -925,7 +896,23 @@ impl PlanWithKeyRequirements {
     }
 }
 
-impl TreeNodeRewritable for PlanWithKeyRequirements {
+impl TreeNode for PlanWithKeyRequirements {
+    fn apply_children<F>(&self, op: &mut F) -> Result<VisitRecursion>
+    where
+        F: FnMut(&Self) -> Result<VisitRecursion>,
+    {
+        let children = self.children();
+        for child in children {
+            match op(&child)? {
+                VisitRecursion::Continue => {}
+                VisitRecursion::Skip => return Ok(VisitRecursion::Continue),
+                VisitRecursion::Stop => return Ok(VisitRecursion::Stop),
+            }
+        }
+
+        Ok(VisitRecursion::Continue)
+    }
+
     fn map_children<F>(self, transform: F) -> Result<Self>
     where
         F: FnMut(Self) -> Result<Self>,
@@ -941,7 +928,7 @@ impl TreeNodeRewritable for PlanWithKeyRequirements {
                 .collect::<Vec<_>>();
             let new_plan = with_new_children_if_necessary(self.plan, children_plans)?;
             Ok(PlanWithKeyRequirements {
-                plan: new_plan,
+                plan: new_plan.into(),
                 required_key_ordering: self.required_key_ordering,
                 request_key_ordering: self.request_key_ordering,
             })
@@ -1015,6 +1002,30 @@ mod tests {
         ))
     }
 
+    // Created a sorted parquet exec with multiple files
+    fn parquet_exec_multiple_sorted(
+        output_ordering: Option<Vec<PhysicalSortExpr>>,
+    ) -> Arc<ParquetExec> {
+        Arc::new(ParquetExec::new(
+            FileScanConfig {
+                object_store_url: ObjectStoreUrl::parse("test:///").unwrap(),
+                file_schema: schema(),
+                file_groups: vec![
+                    vec![PartitionedFile::new("x".to_string(), 100)],
+                    vec![PartitionedFile::new("y".to_string(), 100)],
+                ],
+                statistics: Statistics::default(),
+                projection: None,
+                limit: None,
+                table_partition_cols: vec![],
+                output_ordering,
+                infinite_source: false,
+            },
+            None,
+            None,
+        ))
+    }
+
     fn projection_exec_with_alias(
         input: Arc<dyn ExecutionPlan>,
         alias_pairs: Vec<(String, String)>,
@@ -1657,7 +1668,8 @@ mod tests {
         let bottom_left_join = ensure_distribution(
             hash_join_exec(left.clone(), right.clone(), &join_on, &JoinType::Inner),
             10,
-        )?;
+        )?
+        .into();
 
         // Projection(a as A, a as AA, b as B, c as C)
         let alias_pairs: Vec<(String, String)> = vec![
@@ -1687,7 +1699,8 @@ mod tests {
         let bottom_right_join = ensure_distribution(
             hash_join_exec(left, right.clone(), &join_on, &JoinType::Inner),
             10,
-        )?;
+        )?
+        .into();
 
         // Join on (B == b1 and C == c and AA = a1)
         let top_join_on = vec![
@@ -1776,7 +1789,8 @@ mod tests {
         let bottom_left_join = ensure_distribution(
             hash_join_exec(left.clone(), right.clone(), &join_on, &JoinType::Inner),
             10,
-        )?;
+        )?
+        .into();
 
         // Projection(a as A, a as AA, b as B, c as C)
         let alias_pairs: Vec<(String, String)> = vec![
@@ -1806,7 +1820,8 @@ mod tests {
         let bottom_right_join = ensure_distribution(
             hash_join_exec(left, right.clone(), &join_on, &JoinType::Inner),
             10,
-        )?;
+        )?
+        .into();
 
         // Join on (B == b1 and C == c and AA = a1)
         let top_join_on = vec![
@@ -2093,7 +2108,7 @@ mod tests {
         }];
 
         // Scan some sorted parquet files
-        let exec = parquet_exec_with_sort(Some(sort_key.clone()));
+        let exec = parquet_exec_multiple_sorted(Some(sort_key.clone()));
 
         // CoalesceBatchesExec to mimic behavior after a filter
         let exec = Arc::new(CoalesceBatchesExec::new(exec, 4096));
@@ -2106,7 +2121,7 @@ mod tests {
         let expected = &[
             "SortPreservingMergeExec: [a@0 ASC]",
             "CoalesceBatchesExec: target_batch_size=4096",
-            "ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[a@0 ASC], projection=[a, b, c, d, e]",
+            "ParquetExec: limit=None, partitions={2 groups: [[x], [y]]}, output_ordering=[a@0 ASC], projection=[a, b, c, d, e]",
         ];
         assert_optimized!(expected, exec);
         Ok(())
