@@ -22,8 +22,12 @@
 use crate::config::ConfigOptions;
 use crate::error::Result;
 use crate::physical_optimizer::PhysicalOptimizerRule;
-use crate::physical_plan::tree_node::TreeNodeRewritable;
+use crate::physical_plan::joins::SymmetricHashJoinExec;
 use crate::physical_plan::{with_new_children_if_necessary, ExecutionPlan};
+use datafusion_common::config::OptimizerOptions;
+use datafusion_common::tree_node::{Transformed, TreeNode, VisitRecursion};
+use datafusion_common::DataFusionError;
+use datafusion_physical_expr::intervals::{check_support, is_datatype_supported};
 use std::sync::Arc;
 
 /// The PipelineChecker rule rejects non-runnable query plans that use
@@ -42,10 +46,11 @@ impl PhysicalOptimizerRule for PipelineChecker {
     fn optimize(
         &self,
         plan: Arc<dyn ExecutionPlan>,
-        _config: &ConfigOptions,
+        config: &ConfigOptions,
     ) -> Result<Arc<dyn ExecutionPlan>> {
         let pipeline = PipelineStatePropagator::new(plan);
-        let state = pipeline.transform_up(&check_finiteness_requirements)?;
+        let state = pipeline
+            .transform_up(&|p| check_finiteness_requirements(p, &config.optimizer))?;
         Ok(state.plan)
     }
 
@@ -78,7 +83,23 @@ impl PipelineStatePropagator {
     }
 }
 
-impl TreeNodeRewritable for PipelineStatePropagator {
+impl TreeNode for PipelineStatePropagator {
+    fn apply_children<F>(&self, op: &mut F) -> Result<VisitRecursion>
+    where
+        F: FnMut(&Self) -> Result<VisitRecursion>,
+    {
+        let children = self.plan.children();
+        for child in children {
+            match op(&PipelineStatePropagator::new(child))? {
+                VisitRecursion::Continue => {}
+                VisitRecursion::Skip => return Ok(VisitRecursion::Continue),
+                VisitRecursion::Stop => return Ok(VisitRecursion::Stop),
+            }
+        }
+
+        Ok(VisitRecursion::Continue)
+    }
+
     fn map_children<F>(self, transform: F) -> Result<Self>
     where
         F: FnMut(Self) -> Result<Self>,
@@ -99,7 +120,7 @@ impl TreeNodeRewritable for PipelineStatePropagator {
                 .map(|child| child.plan)
                 .collect::<Vec<_>>();
             Ok(PipelineStatePropagator {
-                plan: with_new_children_if_necessary(self.plan, children_plans)?,
+                plan: with_new_children_if_necessary(self.plan, children_plans)?.into(),
                 unbounded: self.unbounded,
                 children_unbounded,
             })
@@ -112,16 +133,39 @@ impl TreeNodeRewritable for PipelineStatePropagator {
 /// This function propagates finiteness information and rejects any plan with
 /// pipeline-breaking operators acting on infinite inputs.
 pub fn check_finiteness_requirements(
-    input: PipelineStatePropagator,
-) -> Result<Option<PipelineStatePropagator>> {
-    let plan = input.plan;
-    let children = input.children_unbounded;
-    plan.unbounded_output(&children).map(|value| {
-        Some(PipelineStatePropagator {
-            plan,
-            unbounded: value,
-            children_unbounded: children,
+    mut input: PipelineStatePropagator,
+    optimizer_options: &OptimizerOptions,
+) -> Result<Transformed<PipelineStatePropagator>> {
+    if let Some(exec) = input.plan.as_any().downcast_ref::<SymmetricHashJoinExec>() {
+        if !(optimizer_options.allow_symmetric_joins_without_pruning
+            || (exec.check_if_order_information_available()? && is_prunable(exec)))
+        {
+            const MSG: &str = "Join operation cannot operate on a non-prunable stream without enabling \
+                               the 'allow_symmetric_joins_without_pruning' configuration flag";
+            return Err(DataFusionError::Plan(MSG.to_owned()));
+        }
+    }
+    input
+        .plan
+        .unbounded_output(&input.children_unbounded)
+        .map(|value| {
+            input.unbounded = value;
+            Transformed::Yes(input)
         })
+}
+
+/// This function returns whether a given symmetric hash join is amenable to
+/// data pruning. For this to be possible, it needs to have a filter where
+/// all involved [`PhysicalExpr`]s, [`Operator`]s and data types support
+/// interval calculations.
+fn is_prunable(join: &SymmetricHashJoinExec) -> bool {
+    join.filter().map_or(false, |filter| {
+        check_support(filter.expression())
+            && filter
+                .schema()
+                .fields()
+                .iter()
+                .all(|f| is_datatype_supported(f.data_type()))
     })
 }
 
@@ -138,27 +182,19 @@ mod sql_tests {
             source_types: (SourceType::Unbounded, SourceType::Bounded),
             expect_fail: false,
         };
+
         let test2 = BinaryTestCase {
-            source_types: (SourceType::Unbounded, SourceType::Unbounded),
-            expect_fail: true,
-        };
-        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Unbounded),
             expect_fail: true,
         };
-        let test4 = BinaryTestCase {
+        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Bounded),
             expect_fail: false,
         };
         let case = QueryCase {
             sql: "SELECT t2.c1 FROM left as t1 LEFT JOIN right as t2 ON t1.c1 = t2.c1"
                 .to_string(),
-            cases: vec![
-                Arc::new(test1),
-                Arc::new(test2),
-                Arc::new(test3),
-                Arc::new(test4),
-            ],
+            cases: vec![Arc::new(test1), Arc::new(test2), Arc::new(test3)],
             error_operator: "Join Error".to_string(),
         };
 
@@ -173,26 +209,17 @@ mod sql_tests {
             expect_fail: true,
         };
         let test2 = BinaryTestCase {
-            source_types: (SourceType::Unbounded, SourceType::Unbounded),
-            expect_fail: true,
-        };
-        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Unbounded),
             expect_fail: false,
         };
-        let test4 = BinaryTestCase {
+        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Bounded),
             expect_fail: false,
         };
         let case = QueryCase {
             sql: "SELECT t2.c1 FROM left as t1 RIGHT JOIN right as t2 ON t1.c1 = t2.c1"
                 .to_string(),
-            cases: vec![
-                Arc::new(test1),
-                Arc::new(test2),
-                Arc::new(test3),
-                Arc::new(test4),
-            ],
+            cases: vec![Arc::new(test1), Arc::new(test2), Arc::new(test3)],
             error_operator: "Join Error".to_string(),
         };
 
@@ -207,26 +234,17 @@ mod sql_tests {
             expect_fail: false,
         };
         let test2 = BinaryTestCase {
-            source_types: (SourceType::Unbounded, SourceType::Unbounded),
-            expect_fail: true,
-        };
-        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Unbounded),
             expect_fail: false,
         };
-        let test4 = BinaryTestCase {
+        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Bounded),
             expect_fail: false,
         };
         let case = QueryCase {
             sql: "SELECT t2.c1 FROM left as t1 JOIN right as t2 ON t1.c1 = t2.c1"
                 .to_string(),
-            cases: vec![
-                Arc::new(test1),
-                Arc::new(test2),
-                Arc::new(test3),
-                Arc::new(test4),
-            ],
+            cases: vec![Arc::new(test1), Arc::new(test2), Arc::new(test3)],
             error_operator: "Join Error".to_string(),
         };
 
@@ -241,26 +259,17 @@ mod sql_tests {
             expect_fail: true,
         };
         let test2 = BinaryTestCase {
-            source_types: (SourceType::Unbounded, SourceType::Unbounded),
-            expect_fail: true,
-        };
-        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Unbounded),
             expect_fail: true,
         };
-        let test4 = BinaryTestCase {
+        let test3 = BinaryTestCase {
             source_types: (SourceType::Bounded, SourceType::Bounded),
             expect_fail: false,
         };
         let case = QueryCase {
             sql: "SELECT t2.c1 FROM left as t1 FULL JOIN right as t2 ON t1.c1 = t2.c1"
                 .to_string(),
-            cases: vec![
-                Arc::new(test1),
-                Arc::new(test2),
-                Arc::new(test3),
-                Arc::new(test4),
-            ],
+            cases: vec![Arc::new(test1), Arc::new(test2), Arc::new(test3)],
             error_operator: "Join Error".to_string(),
         };
 
@@ -305,7 +314,7 @@ mod sql_tests {
                   FROM test
                   LIMIT 5".to_string(),
             cases: vec![Arc::new(test1), Arc::new(test2)],
-            error_operator: "Sort Error".to_string()
+            error_operator: "Window Error".to_string()
         };
 
         case.run().await?;
@@ -328,7 +337,7 @@ mod sql_tests {
                         SUM(c9) OVER(ORDER BY c9 ASC ROWS BETWEEN 1 PRECEDING AND UNBOUNDED FOLLOWING) as sum1
                   FROM test".to_string(),
             cases: vec![Arc::new(test1), Arc::new(test2)],
-            error_operator: "Sort Error".to_string()
+            error_operator: "Window Error".to_string()
         };
         case.run().await?;
         Ok(())
