@@ -45,8 +45,8 @@ use arrow::{
     datatypes::{DataType, Field, Schema, SchemaRef},
     record_batch::RecordBatch,
 };
+use datafusion_common::tree_node::{Transformed, TreeNode};
 use datafusion_common::{downcast_value, ScalarValue};
-use datafusion_physical_expr::rewrite::TreeNodeRewritable;
 use datafusion_physical_expr::utils::collect_columns;
 use datafusion_physical_expr::{expressions as phys_expr, PhysicalExprRef};
 use log::trace;
@@ -384,7 +384,7 @@ fn build_statistics_record_batch<S: PruningStatistics>(
     let mut arrays = Vec::<ArrayRef>::new();
     // For each needed statistics column:
     for (column, statistics_type, stat_field) in required_columns.iter() {
-        let column = Column::from_qualified_name(column.name());
+        let column = Column::from_name(column.name());
         let data_type = stat_field.data_type();
 
         let num_containers = statistics.num_containers();
@@ -646,11 +646,11 @@ fn rewrite_column_expr(
     e.transform(&|expr| {
         if let Some(column) = expr.as_any().downcast_ref::<phys_expr::Column>() {
             if column == column_old {
-                return Ok(Some(Arc::new(column_new.clone())));
+                return Ok(Transformed::Yes(Arc::new(column_new.clone())));
             }
         }
 
-        Ok(None)
+        Ok(Transformed::No(expr))
     })
 }
 
