@@ -16,6 +16,7 @@
 // under the License.
 
 //! Repartition optimizer that introduces repartition nodes to increase the level of parallelism available
+use datafusion_common::tree_node::Transformed;
 use std::sync::Arc;
 
 use super::optimizer::PhysicalOptimizerRule;
@@ -170,12 +171,12 @@ fn optimize_partitions(
     would_benefit: bool,
     repartition_file_scans: bool,
     repartition_file_min_size: usize,
-) -> Result<Arc<dyn ExecutionPlan>> {
+) -> Result<Transformed<Arc<dyn ExecutionPlan>>> {
     // Recurse into children bottom-up (attempt to repartition as
     // early as possible)
     let new_plan = if plan.children().is_empty() {
         // leaf node - don't replace children
-        plan
+        Transformed::No(plan)
     } else {
         let children = plan
             .children()
@@ -205,11 +206,14 @@ fn optimize_partitions(
                     repartition_file_scans,
                     repartition_file_min_size,
                 )
+                .map(Transformed::into)
             })
             .collect::<Result<_>>()?;
         with_new_children_if_necessary(plan, children)?
     };
 
+    let (new_plan, transformed) = new_plan.into_pair();
+
     // decide if we should bother trying to repartition the output of this plan
     let mut could_repartition = match new_plan.output_partitioning() {
         // Apply when underlying node has less than `self.target_partitions` amount of concurrency
@@ -236,24 +240,28 @@ fn optimize_partitions(
 
     // If repartition is not allowed - return plan as it is
     if !repartition_allowed {
-        return Ok(new_plan);
+        return Ok(if transformed {
+            Transformed::Yes(new_plan)
+        } else {
+            Transformed::No(new_plan)
+        });
     }
 
     // For ParquetExec return internally repartitioned version of the plan in case `repartition_file_scans` is set
     if let Some(parquet_exec) = new_plan.as_any().downcast_ref::<ParquetExec>() {
         if repartition_file_scans {
-            return Ok(Arc::new(
+            return Ok(Transformed::Yes(Arc::new(
                 parquet_exec
                     .get_repartitioned(target_partitions, repartition_file_min_size),
-            ));
+            )));
         }
     }
 
     // Otherwise - return plan wrapped up in RepartitionExec
-    Ok(Arc::new(RepartitionExec::try_new(
+    Ok(Transformed::Yes(Arc::new(RepartitionExec::try_new(
         new_plan,
         RoundRobinBatch(target_partitions),
-    )?))
+    )?)))
 }
 
 /// Returns true if `plan` requires any of inputs to be sorted in some
@@ -290,6 +298,7 @@ impl PhysicalOptimizerRule for Repartition {
                 repartition_file_scans,
                 repartition_file_min_size,
             )
+            .map(Transformed::into)
         }
     }
 
@@ -330,6 +339,7 @@ mod tests {
     use crate::physical_plan::sorts::sort_preserving_merge::SortPreservingMergeExec;
     use crate::physical_plan::union::UnionExec;
     use crate::physical_plan::{displayable, DisplayFormatType, Statistics};
+    use datafusion_physical_expr::PhysicalSortRequirement;
 
     fn schema() -> SchemaRef {
         Arc::new(Schema::new(vec![Field::new("c1", DataType::Boolean, true)]))
@@ -400,6 +410,33 @@ mod tests {
         ))
     }
 
+    // Created a sorted parquet exec with multiple files
+    fn parquet_exec_multiple_sorted() -> Arc<ParquetExec> {
+        let sort_exprs = vec![PhysicalSortExpr {
+            expr: col("c1", &schema()).unwrap(),
+            options: SortOptions::default(),
+        }];
+
+        Arc::new(ParquetExec::new(
+            FileScanConfig {
+                object_store_url: ObjectStoreUrl::parse("test:///").unwrap(),
+                file_schema: schema(),
+                file_groups: vec![
+                    vec![PartitionedFile::new("x".to_string(), 100)],
+                    vec![PartitionedFile::new("y".to_string(), 100)],
+                ],
+                statistics: Statistics::default(),
+                projection: None,
+                limit: None,
+                table_partition_cols: vec![],
+                output_ordering: Some(sort_exprs),
+                infinite_source: false,
+            },
+            None,
+            None,
+        ))
+    }
+
     fn sort_preserving_merge_exec(
         input: Arc<dyn ExecutionPlan>,
     ) -> Arc<dyn ExecutionPlan> {
@@ -725,12 +762,12 @@ mod tests {
     #[test]
     fn repartition_ignores_sort_preserving_merge() -> Result<()> {
         // sort preserving merge already sorted input,
-        let plan = sort_preserving_merge_exec(parquet_exec_sorted());
+        let plan = sort_preserving_merge_exec(parquet_exec_multiple_sorted());
 
         // should not repartition / sort (as the data was already sorted)
         let expected = &[
             "SortPreservingMergeExec: [c1@0 ASC]",
-            "ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[c1@0 ASC], projection=[c1]",
+            "ParquetExec: limit=None, partitions={2 groups: [[x], [y]]}, output_ordering=[c1@0 ASC], projection=[c1]",
         ];
 
         assert_optimized!(expected, plan);
@@ -826,13 +863,14 @@ mod tests {
     #[test]
     fn repartition_ignores_transitively_with_projection() -> Result<()> {
         // sorted input
-        let plan = sort_preserving_merge_exec(projection_exec(parquet_exec_sorted()));
+        let plan =
+            sort_preserving_merge_exec(projection_exec(parquet_exec_multiple_sorted()));
 
         // data should not be repartitioned / resorted
         let expected = &[
             "SortPreservingMergeExec: [c1@0 ASC]",
             "ProjectionExec: expr=[c1@0 as c1]",
-            "ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[c1@0 ASC], projection=[c1]",
+            "ParquetExec: limit=None, partitions={2 groups: [[x], [y]]}, output_ordering=[c1@0 ASC], projection=[c1]",
         ];
 
         assert_optimized!(expected, plan);
@@ -845,7 +883,6 @@ mod tests {
             sort_preserving_merge_exec(sort_exec(projection_exec(parquet_exec()), true));
 
         let expected = &[
-            "SortPreservingMergeExec: [c1@0 ASC]",
             "SortExec: expr=[c1@0 ASC]",
             "ProjectionExec: expr=[c1@0 as c1]",
             "ParquetExec: limit=None, partitions={1 group: [[x]]}, projection=[c1]",
@@ -1028,7 +1065,6 @@ mod tests {
 
         // parallelization potentially could break sort order
         let expected = &[
-            "SortPreservingMergeExec: [c1@0 ASC]",
             "ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[c1@0 ASC], projection=[c1]",
         ];
 
@@ -1078,7 +1114,6 @@ mod tests {
 
         // data should not be repartitioned / resorted
         let expected = &[
-            "SortPreservingMergeExec: [c1@0 ASC]",
             "ProjectionExec: expr=[c1@0 as c1]",
             "ParquetExec: limit=None, partitions={1 group: [[x]]}, output_ordering=[c1@0 ASC], projection=[c1]",
         ];
@@ -1122,8 +1157,10 @@ mod tests {
         }
 
         // model that it requires the output ordering of its input
-        fn required_input_ordering(&self) -> Vec<Option<&[PhysicalSortExpr]>> {
-            vec![self.input.output_ordering()]
+        fn required_input_ordering(&self) -> Vec<Option<Vec<PhysicalSortRequirement>>> {
+            vec![self
+                .output_ordering()
+                .map(PhysicalSortRequirement::from_sort_exprs)]
         }
 
         fn with_new_children(
