@@ -73,6 +73,11 @@ impl NdJsonExec {
             file_compression_type,
         }
     }
+
+    /// Ref to the base configs
+    pub fn base_config(&self) -> &FileScanConfig {
+        &self.base_config
+    }
 }
 
 impl ExecutionPlan for NdJsonExec {
@@ -326,8 +331,8 @@ mod tests {
         store: Arc<dyn ObjectStore>,
     ) {
         let ctx = SessionContext::new();
-        ctx.runtime_env()
-            .register_object_store("file", "", store.clone());
+        let url = Url::parse("file://").unwrap();
+        ctx.runtime_env().register_object_store(&url, store.clone());
         let filename = "1.json";
         let file_groups = partitioned_file_groups(
             TEST_DATA_BASE,
