@@ -45,9 +45,6 @@ use crate::datasource::{
     listing::{FileRange, PartitionedFile},
     object_store::ObjectStoreUrl,
 };
-use crate::physical_plan::tree_node::{
-    TreeNodeVisitable, TreeNodeVisitor, VisitRecursion,
-};
 use crate::physical_plan::ExecutionPlan;
 use crate::{
     error::{DataFusionError, Result},
@@ -55,6 +52,7 @@ use crate::{
 };
 use arrow::array::new_null_array;
 use arrow::record_batch::RecordBatchOptions;
+use datafusion_common::tree_node::{TreeNode, VisitRecursion};
 use log::{debug, info, warn};
 use object_store::path::Path;
 use object_store::ObjectMeta;
@@ -94,28 +92,9 @@ pub fn wrap_partition_value_in_dict(val: ScalarValue) -> ScalarValue {
 pub fn get_scan_files(
     plan: Arc<dyn ExecutionPlan>,
 ) -> Result<Vec<Vec<Vec<PartitionedFile>>>> {
-    let mut collector = FileScanCollector::new();
-    plan.accept(&mut collector)?;
-    Ok(collector.file_groups)
-}
-
-struct FileScanCollector {
-    file_groups: Vec<Vec<Vec<PartitionedFile>>>,
-}
-
-impl FileScanCollector {
-    fn new() -> Self {
-        Self {
-            file_groups: vec![],
-        }
-    }
-}
-
-impl TreeNodeVisitor for FileScanCollector {
-    type N = Arc<dyn ExecutionPlan>;
-
-    fn pre_visit(&mut self, node: &Self::N) -> Result<VisitRecursion> {
-        let plan_any = node.as_any();
+    let mut collector: Vec<Vec<Vec<PartitionedFile>>> = vec![];
+    plan.apply(&mut |plan| {
+        let plan_any = plan.as_any();
         let file_groups =
             if let Some(parquet_exec) = plan_any.downcast_ref::<ParquetExec>() {
                 parquet_exec.base_config().file_groups.clone()
@@ -129,9 +108,10 @@ impl TreeNodeVisitor for FileScanCollector {
                 return Ok(VisitRecursion::Continue);
             };
 
-        self.file_groups.push(file_groups);
-        Ok(VisitRecursion::Stop)
-    }
+        collector.push(file_groups);
+        Ok(VisitRecursion::Skip)
+    })?;
+    Ok(collector)
 }
 
 /// The base configurations to provide when creating a physical plan for
