@@ -398,10 +398,10 @@ async fn test_fn_regexp_match() -> Result<()> {
         "+-----------------------------------+",
         "| regexpmatch(test.a,Utf8(\"[a-z]\")) |",
         "+-----------------------------------+",
-        "| []                                |",
-        "| []                                |",
-        "| []                                |",
-        "| []                                |",
+        "| [a]                               |",
+        "| [a]                               |",
+        "| [d]                               |",
+        "| [b]                               |",
         "+-----------------------------------+",
     ];
 
