@@ -20,6 +20,7 @@
 use std::sync::Arc;
 
 use datafusion::datasource::MemTable;
+use datafusion::execution::context::SessionState;
 use datafusion::execution::disk_manager::DiskManagerConfig;
 use datafusion::execution::runtime_env::{RuntimeConfig, RuntimeEnv};
 use datafusion_common::assert_contains;
@@ -37,7 +38,9 @@ fn init() {
 async fn oom_sort() {
     run_limit_test(
         "select * from t order by host DESC",
-        "Resources exhausted: Memory Exhausted while Sorting (DiskManager is disabled)",
+        vec![
+            "Resources exhausted: Memory Exhausted while Sorting (DiskManager is disabled)",
+        ],
         200_000,
     )
     .await
@@ -47,7 +50,10 @@ async fn oom_sort() {
 async fn group_by_none() {
     run_limit_test(
         "select median(image) from t",
-        "Resources exhausted: Failed to allocate additional",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "AggregateStream",
+        ],
         20_000,
     )
     .await
@@ -57,7 +63,10 @@ async fn group_by_none() {
 async fn group_by_row_hash() {
     run_limit_test(
         "select count(*) from t GROUP BY response_bytes",
-        "Resources exhausted: Failed to allocate additional",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "GroupedHashAggregateStream",
+        ],
         2_000,
     )
     .await
@@ -68,17 +77,53 @@ async fn group_by_hash() {
     run_limit_test(
         // group by dict column
         "select count(*) from t GROUP BY service, host, pod, container",
-        "Resources exhausted: Failed to allocate additional",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "GroupedHashAggregateStream",
+        ],
         1_000,
     )
     .await
 }
 
 #[tokio::test]
-async fn join_by_key() {
-    run_limit_test(
+async fn join_by_key_multiple_partitions() {
+    let config = SessionConfig::new().with_target_partitions(2);
+    run_limit_test_with_config(
         "select t1.* from t t1 JOIN t t2 ON t1.service = t2.service",
-        "Resources exhausted: Failed to allocate additional",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "HashJoinStream",
+        ],
+        1_000,
+        config,
+    )
+    .await
+}
+
+#[tokio::test]
+async fn join_by_key_single_partition() {
+    let config = SessionConfig::new().with_target_partitions(1);
+    run_limit_test_with_config(
+        "select t1.* from t t1 JOIN t t2 ON t1.service = t2.service",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "HashJoinExec",
+        ],
+        1_000,
+        config,
+    )
+    .await
+}
+
+#[tokio::test]
+async fn join_by_expression() {
+    run_limit_test(
+        "select t1.* from t t1 JOIN t t2 ON t1.service != t2.service",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "NestedLoopJoinExec",
+        ],
         1_000,
     )
     .await
@@ -88,18 +133,58 @@ async fn join_by_key() {
 async fn cross_join() {
     run_limit_test(
         "select t1.* from t t1 CROSS JOIN t t2",
-        "Resources exhausted: Failed to allocate additional",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "CrossJoinExec",
+        ],
         1_000,
     )
     .await
 }
 
+#[tokio::test]
+async fn merge_join() {
+    // Planner chooses MergeJoin only if number of partitions > 1
+    let config = SessionConfig::new()
+        .with_target_partitions(2)
+        .set_bool("datafusion.optimizer.prefer_hash_join", false);
+
+    run_limit_test_with_config(
+        "select t1.* from t t1 JOIN t t2 ON t1.pod = t2.pod AND t1.time = t2.time",
+        vec![
+            "Resources exhausted: Failed to allocate additional",
+            "SMJStream",
+        ],
+        1_000,
+        config,
+    )
+    .await
+}
+
 /// 50 byte memory limit
 const MEMORY_FRACTION: f64 = 0.95;
 
 /// runs the specified query against 1000 rows with a 50
-/// byte memory limit and no disk manager enabled.
-async fn run_limit_test(query: &str, expected_error: &str, memory_limit: usize) {
+/// byte memory limit and no disk manager enabled
+/// with default SessionConfig.
+async fn run_limit_test(
+    query: &str,
+    expected_error_contains: Vec<&str>,
+    memory_limit: usize,
+) {
+    let config = SessionConfig::new();
+    run_limit_test_with_config(query, expected_error_contains, memory_limit, config).await
+}
+
+/// runs the specified query against 1000 rows with a 50
+/// byte memory limit and no disk manager enabled
+/// with specified SessionConfig instance
+async fn run_limit_test_with_config(
+    query: &str,
+    expected_error_contains: Vec<&str>,
+    memory_limit: usize,
+    config: SessionConfig,
+) {
     let batches: Vec<_> = AccessLogGenerator::new()
         .with_row_limit(1000)
         .with_max_batch_size(50)
@@ -115,11 +200,12 @@ async fn run_limit_test(query: &str, expected_error: &str, memory_limit: usize)
 
     let runtime = RuntimeEnv::new(rt_config).unwrap();
 
-    let ctx = SessionContext::with_config_rt(
-        // do NOT re-partition (since RepartitionExec has also has a memory budget which we'll likely hit first)
-        SessionConfig::new().with_target_partitions(1),
-        Arc::new(runtime),
-    );
+    // Disabling physical optimizer rules to avoid sorts / repartitions
+    // (since RepartitionExec / SortExec also has a memory budget which we'll likely hit first)
+    let state = SessionState::with_config_rt(config, Arc::new(runtime))
+        .with_physical_optimizer_rules(vec![]);
+
+    let ctx = SessionContext::with_state(state);
     ctx.register_table("t", Arc::new(table))
         .expect("registering table");
 
@@ -130,7 +216,9 @@ async fn run_limit_test(query: &str, expected_error: &str, memory_limit: usize)
             panic!("Unexpected success when running, expected memory limit failure")
         }
         Err(e) => {
-            assert_contains!(e.to_string(), expected_error);
+            for error_substring in expected_error_contains {
+                assert_contains!(e.to_string(), error_substring);
+            }
         }
     }
 }
