@@ -26,25 +26,25 @@ async fn select_qualified_wildcard() -> Result<()> {
     let results = execute_to_batches(&ctx, sql).await;
 
     let expected = vec![
-        "+---------+----------------+-------+",
-        "| c1      | c2             | c3    |",
-        "+---------+----------------+-------+",
-        "| 0.00001 | 0.000000000001 | true  |",
-        "| 0.00002 | 0.000000000002 | false |",
-        "| 0.00002 | 0.000000000002 | false |",
-        "| 0.00003 | 0.000000000003 | true  |",
-        "| 0.00003 | 0.000000000003 | true  |",
-        "| 0.00003 | 0.000000000003 | true  |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "+---------+----------------+-------+",
+        "+---------+---------+-------+",
+        "| c1      | c2      | c3    |",
+        "+---------+---------+-------+",
+        "| 0.00001 | 1.0e-12 | true  |",
+        "| 0.00002 | 2.0e-12 | false |",
+        "| 0.00002 | 2.0e-12 | false |",
+        "| 0.00003 | 3.0e-12 | true  |",
+        "| 0.00003 | 3.0e-12 | true  |",
+        "| 0.00003 | 3.0e-12 | true  |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "+---------+---------+-------+",
     ];
 
     assert_batches_eq!(expected, &results);
@@ -61,25 +61,25 @@ async fn select_non_alias_qualified_wildcard() -> Result<()> {
     let results = execute_to_batches(&ctx, sql).await;
 
     let expected = vec![
-        "+---------+----------------+-------+",
-        "| c1      | c2             | c3    |",
-        "+---------+----------------+-------+",
-        "| 0.00001 | 0.000000000001 | true  |",
-        "| 0.00002 | 0.000000000002 | false |",
-        "| 0.00002 | 0.000000000002 | false |",
-        "| 0.00003 | 0.000000000003 | true  |",
-        "| 0.00003 | 0.000000000003 | true  |",
-        "| 0.00003 | 0.000000000003 | true  |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00004 | 0.000000000004 | false |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "| 0.00005 | 0.000000000005 | true  |",
-        "+---------+----------------+-------+",
+        "+---------+---------+-------+",
+        "| c1      | c2      | c3    |",
+        "+---------+---------+-------+",
+        "| 0.00001 | 1.0e-12 | true  |",
+        "| 0.00002 | 2.0e-12 | false |",
+        "| 0.00002 | 2.0e-12 | false |",
+        "| 0.00003 | 3.0e-12 | true  |",
+        "| 0.00003 | 3.0e-12 | true  |",
+        "| 0.00003 | 3.0e-12 | true  |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00004 | 4.0e-12 | false |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "| 0.00005 | 5.0e-12 | true  |",
+        "+---------+---------+-------+",
     ];
 
     assert_batches_eq!(expected, &results);
