@@ -733,7 +733,7 @@ mod tests {
                 vec![
                     DFField::new(Some("test"), "a", DataType::UInt32, false),
                     DFField::new(Some("test"), "b", DataType::UInt32, false),
-                    DFField::new(Some("test2"), "c1", DataType::UInt32, false),
+                    DFField::new(Some("test2"), "c1", DataType::UInt32, true),
                 ],
                 HashMap::new(),
             )?,
@@ -776,7 +776,7 @@ mod tests {
                 vec![
                     DFField::new(Some("test"), "a", DataType::UInt32, false),
                     DFField::new(Some("test"), "b", DataType::UInt32, false),
-                    DFField::new(Some("test2"), "c1", DataType::UInt32, false),
+                    DFField::new(Some("test2"), "c1", DataType::UInt32, true),
                 ],
                 HashMap::new(),
             )?,
@@ -817,7 +817,7 @@ mod tests {
                 vec![
                     DFField::new(Some("test"), "a", DataType::UInt32, false),
                     DFField::new(Some("test"), "b", DataType::UInt32, false),
-                    DFField::new(Some("test2"), "a", DataType::UInt32, false),
+                    DFField::new(Some("test2"), "a", DataType::UInt32, true),
                 ],
                 HashMap::new(),
             )?,
