@@ -18,24 +18,23 @@
 //! Expression simplification API
 
 use super::utils::*;
-use crate::{
-    simplify_expressions::regex::simplify_regex_expr, type_coercion::TypeCoercionRewriter,
-};
+use crate::analyzer::type_coercion::TypeCoercionRewriter;
+use crate::simplify_expressions::regex::simplify_regex_expr;
 use arrow::{
     array::new_null_array,
     datatypes::{DataType, Field, Schema},
     error::ArrowError,
     record_batch::RecordBatch,
 };
+use datafusion_common::tree_node::{RewriteRecursion, TreeNode, TreeNodeRewriter};
 use datafusion_common::{DFSchema, DFSchemaRef, DataFusionError, Result, ScalarValue};
 use datafusion_expr::{
-    and,
-    expr_rewriter::{ExprRewritable, ExprRewriter, RewriteRecursion},
-    lit, or, BinaryExpr, BuiltinScalarFunction, ColumnarValue, Expr, Volatility,
+    and, lit, or, BinaryExpr, BuiltinScalarFunction, ColumnarValue, Expr, Like,
+    Volatility,
 };
 use datafusion_physical_expr::{create_physical_expr, execution_props::ExecutionProps};
 
-use super::SimplifyInfo;
+use crate::simplify_expressions::SimplifyInfo;
 
 /// This structure handles API for expression simplification
 pub struct ExprSimplifier<S> {
@@ -168,7 +167,9 @@ struct ConstEvaluator<'a> {
     input_batch: RecordBatch,
 }
 
-impl<'a> ExprRewriter for ConstEvaluator<'a> {
+impl<'a> TreeNodeRewriter for ConstEvaluator<'a> {
+    type N = Expr;
+
     fn pre_visit(&mut self, expr: &Expr) -> Result<RewriteRecursion> {
         // Default to being able to evaluate this node
         self.can_evaluate.push(true);
@@ -338,7 +339,9 @@ impl<'a, S> Simplifier<'a, S> {
     }
 }
 
-impl<'a, S: SimplifyInfo> ExprRewriter for Simplifier<'a, S> {
+impl<'a, S: SimplifyInfo> TreeNodeRewriter for Simplifier<'a, S> {
+    type N = Expr;
+
     /// rewrite the expression simplifying any constant expressions
     fn mutate(&mut self, expr: Expr) -> Result<Expr> {
         use datafusion_expr::Operator::{
@@ -1069,6 +1072,18 @@ impl<'a, S: SimplifyInfo> ExprRewriter for Simplifier<'a, S> {
                 out_expr.rewrite(self)?
             }
 
+            // log
+            Expr::ScalarFunction {
+                fun: BuiltinScalarFunction::Log,
+                args,
+            } => simpl_log(args, <&S>::clone(&info))?,
+
+            // power
+            Expr::ScalarFunction {
+                fun: BuiltinScalarFunction::Power,
+                args,
+            } => simpl_power(args, <&S>::clone(&info))?,
+
             // concat
             Expr::ScalarFunction {
                 fun: BuiltinScalarFunction::Concat,
@@ -1115,6 +1130,36 @@ impl<'a, S: SimplifyInfo> ExprRewriter for Simplifier<'a, S> {
                 right,
             }) => simplify_regex_expr(left, op, right)?,
 
+            // Rules for Like
+            Expr::Like(Like {
+                expr,
+                pattern,
+                negated,
+                escape_char: _,
+            }) if !is_null(&expr)
+                && matches!(
+                    pattern.as_ref(),
+                    Expr::Literal(ScalarValue::Utf8(Some(pattern_str))) if pattern_str == "%"
+                ) =>
+            {
+                lit(!negated)
+            }
+
+            // Rules for ILike
+            Expr::ILike(Like {
+                expr,
+                pattern,
+                negated,
+                escape_char: _,
+            }) if !is_null(&expr)
+                && matches!(
+                    pattern.as_ref(),
+                    Expr::Literal(ScalarValue::Utf8(Some(pattern_str))) if pattern_str == "%"
+                ) =>
+            {
+                lit(!negated)
+            }
+
             // no additional rewrites possible
             expr => expr,
         };
@@ -2207,6 +2252,68 @@ mod tests {
         assert_eq!(simplify(expr_eq), lit(true));
     }
 
+    #[test]
+    fn test_simplify_log() {
+        // Log(c3, 1) ===> 0
+        {
+            let expr = log(col("c3_non_null"), lit(1));
+            let expected = lit(0i64);
+            assert_eq!(simplify(expr), expected);
+        }
+        // Log(c3, c3) ===> 1
+        {
+            let expr = log(col("c3_non_null"), col("c3_non_null"));
+            let expected = lit(1i64);
+            assert_eq!(simplify(expr), expected);
+        }
+        // Log(c3, Power(c3, c4)) ===> c4
+        {
+            let expr = log(
+                col("c3_non_null"),
+                power(col("c3_non_null"), col("c4_non_null")),
+            );
+            let expected = col("c4_non_null");
+            assert_eq!(simplify(expr), expected);
+        }
+        // Log(c3, c4) ===> Log(c3, c4)
+        {
+            let expr = log(col("c3_non_null"), col("c4_non_null"));
+            let expected = log(col("c3_non_null"), col("c4_non_null"));
+            assert_eq!(simplify(expr), expected);
+        }
+    }
+
+    #[test]
+    fn test_simplify_power() {
+        // Power(c3, 0) ===> 1
+        {
+            let expr = power(col("c3_non_null"), lit(0));
+            let expected = lit(1i64);
+            assert_eq!(simplify(expr), expected);
+        }
+        // Power(c3, 1) ===> c3
+        {
+            let expr = power(col("c3_non_null"), lit(1));
+            let expected = col("c3_non_null");
+            assert_eq!(simplify(expr), expected);
+        }
+        // Power(c3, Log(c3, c4)) ===> c4
+        {
+            let expr = power(
+                col("c3_non_null"),
+                log(col("c3_non_null"), col("c4_non_null")),
+            );
+            let expected = col("c4_non_null");
+            assert_eq!(simplify(expr), expected);
+        }
+        // Power(c3, c4) ===> Power(c3, c4)
+        {
+            let expr = power(col("c3_non_null"), col("c4_non_null"));
+            let expected = power(col("c3_non_null"), col("c4_non_null"));
+            assert_eq!(simplify(expr), expected);
+        }
+    }
+
     #[test]
     fn test_simplify_concat_ws() {
         let null = lit(ScalarValue::Utf8(None));
@@ -2317,16 +2424,10 @@ mod tests {
         assert_no_change(regex_match(col("c1"), lit("f_o")));
 
         // empty cases
-        assert_change(regex_match(col("c1"), lit("")), like(col("c1"), "%"));
-        assert_change(
-            regex_not_match(col("c1"), lit("")),
-            not_like(col("c1"), "%"),
-        );
-        assert_change(regex_imatch(col("c1"), lit("")), ilike(col("c1"), "%"));
-        assert_change(
-            regex_not_imatch(col("c1"), lit("")),
-            not_ilike(col("c1"), "%"),
-        );
+        assert_change(regex_match(col("c1"), lit("")), lit(true));
+        assert_change(regex_not_match(col("c1"), lit("")), lit(false));
+        assert_change(regex_imatch(col("c1"), lit("")), lit(true));
+        assert_change(regex_not_imatch(col("c1"), lit("")), lit(false));
 
         // single character
         assert_change(regex_match(col("c1"), lit("x")), like(col("c1"), "%x%"));
@@ -2347,12 +2448,6 @@ mod tests {
                 .or(like(col("c1"), "%x%"))
                 .or(like(col("c1"), "%baz%")),
         );
-        assert_change(
-            regex_match(col("c1"), lit("foo||baz")),
-            like(col("c1"), "%foo%")
-                .or(like(col("c1"), "%"))
-                .or(like(col("c1"), "%baz%")),
-        );
         assert_change(
             regex_not_match(col("c1"), lit("foo|bar|baz")),
             not_like(col("c1"), "%foo%")
@@ -2848,4 +2943,34 @@ mod tests {
             or(col("c2").lt(lit(3)), col("c2").gt(lit(4)))
         );
     }
+
+    #[test]
+    fn test_like_and_ilke() {
+        // test non-null values
+        let expr = like(col("c1"), "%");
+        assert_eq!(simplify(expr), lit(true));
+
+        let expr = not_like(col("c1"), "%");
+        assert_eq!(simplify(expr), lit(false));
+
+        let expr = ilike(col("c1"), "%");
+        assert_eq!(simplify(expr), lit(true));
+
+        let expr = not_ilike(col("c1"), "%");
+        assert_eq!(simplify(expr), lit(false));
+
+        // test null values
+        let null = lit(ScalarValue::Utf8(None));
+        let expr = like(null.clone(), "%");
+        assert_eq!(simplify(expr), lit_bool_null());
+
+        let expr = not_like(null.clone(), "%");
+        assert_eq!(simplify(expr), lit_bool_null());
+
+        let expr = ilike(null.clone(), "%");
+        assert_eq!(simplify(expr), lit_bool_null());
+
+        let expr = not_ilike(null, "%");
+        assert_eq!(simplify(expr), lit_bool_null());
+    }
 }
