@@ -143,6 +143,10 @@ impl Accumulator for MedianAccumulator {
     }
 
     fn evaluate(&self) -> Result<ScalarValue> {
+        if !self.all_values.iter().any(|v| !v.is_null()) {
+            return ScalarValue::try_from(&self.data_type);
+        }
+
         // Create an array of all the non null values and find the
         // sorted indexes
         let array = ScalarValue::iter_to_array(
