@@ -22,5 +22,5 @@ pub mod cp_solver;
 pub mod interval_aritmetic;
 
 pub mod test_utils;
-pub use cp_solver::ExprIntervalGraph;
+pub use cp_solver::{check_support, ExprIntervalGraph};
 pub use interval_aritmetic::*;
