@@ -386,11 +386,8 @@ mod tests {
     #[test]
     #[should_panic(expected = "not supported yet")]
     fn test_unsupported_type_read() {
-        let schema = Arc::new(Schema::new(vec![Field::new(
-            "a",
-            DataType::Decimal128(5, 2),
-            false,
-        )]));
+        let schema =
+            Arc::new(Schema::new(vec![Field::new("a", Decimal128(5, 2), false)]));
         let vector = vec![0; 1024];
         let row_offsets = vec![0];
         read_as_batch(&vector, schema, &row_offsets, Compact).unwrap();
